var LEFT_MARGIN = 230;
var BTN_HEIGHT = 50;
var BTN_WIDTH = 190;
var BTN_MARGIN_V = 10;
var TOP_MARGIN_BTN = 130;
var TOP_MARGIN = 28;

var ROY_THUMB = new Sprite("../img/mascot.png");
var ROY_MEGA = new Sprite("../img/mascot_megaphone.png");
var ROY_MICRO = new Sprite("../img/mascot_microphone.png");
var ROY = ROY_THUMB;
var ROY_POS = new Vector2(-40, 200);
var ROY_SCALE = 0.5;
var ROY_CIRCLE = new Sprite("../img/mascot_circle.png");
var ROY_CIRCLE_POS = new Vector2();
var ROY_CIRCLE_SCALE = 0.4;

var BACKGROUND = new Sprite("../img/background.png");
var BACKGROUND_POS = new Vector2(0, 0);

var SIGN = new Sprite("../img/sign.png");
var SIGN_POS = new Vector2(20, 106);
var SIGN_SCALE = 0.75;
var SIGN_NAME = new Vector2(47, 160);
var SIGN_WIDTH = 110;

var SPOTLIGHT = new Sprite("../img/spotlight.png");
var SPOTLIGHT_POS = new Vector2(-12, SIGN_POS.y);
var SPOTLIGHT_SCALE = 1.0;
var LIGHTBEAM = new Sprite("../img/lightbeam.png");
var LIGHTBEAM_POS = SPOTLIGHT_POS;
var LIGHTBEAM_SCALE = 1.0;

var SAVE_ICON = new Sprite("../img/save.png");
var SAVE_ICON_POS = new Vector2(775, 10);

var POPOVER_SIZE = new Vector2(222, 113);
var POPOVER_ALPHA = 0.9;

var ORIG_BTN_SELL_COLOUR = "#735716";

var RENDER_DEBUG_TEXT = false;

var cookieManager;

function PlayState() {
    "use strict";

    this.status = new Status();
    this.shopName = undefined;
    this.sign = false;
    var isChain = false;
    var makesCommercials = false;

    //////////////////////////////////////////////////
    ///             BUTTON CREATION                ///
    //////////////////////////////////////////////////

    var self = this;

    var createSmallButton = function (name, price, parent, text, onclick) {
        var poData = {
            size: POPOVER_SIZE,
            title: name,
            titleSize: 16,
            price: price,
            text: text,
            textSize: 14,
            bgColour: "#000",
            textColour: "#FFF"
        };
        var upgrade = new UpgradeButton(undefined, undefined,
            "#FD5",
            undefined,
            name,
            price,
            16, "#540", undefined,
            "#FA5", 1,
            poData);
        parent.addUpgrade(upgrade);
        upgrade.btn.onclick = function () {
            onclick(upgrade);
        }
        return upgrade;
    }

    var createBigButton = function (index, object, price, text, onclick) {
        if (text !== undefined) {
            var poData = {
                size: POPOVER_SIZE,
                title: ("Buy " + object),
                titleSize: 16,
                price: price,
                text: text,
                textSize: 14,
                bgColour: "#000",
                textColour: "#FFF"
            };
        }
        var upgrade = new UpgradeButton(
            new Vector2(LEFT_MARGIN,
                TOP_MARGIN_BTN + (BTN_HEIGHT + BTN_MARGIN_V) * index),
            new Vector2(BTN_WIDTH, BTN_HEIGHT),
            "#55F",
            "#1C1CA6",
            object,
            price,
            undefined, undefined, undefined,
            "#5517A6", 1,
            poData);
        upgrade.btn.onclick = function () {
            onclick(upgrade);
        }
        return upgrade;
    }

    //////////////////////////////////////////////////
    ///            HELPER FUNCTIONS                ///
    //////////////////////////////////////////////////

    var upgradeShoutStrength = function (modifier) {
        if (typeof modifier === 'undefined') {
            modifier = 1;
        }
        self.status.shoutStrength += self.status.shoutStrengthGain * modifier;
        self.status.shoutStrengthGain *= 1.5;
    };

    var upgradeMinimumCustomers = function (modifier) {
        if (typeof modifier === 'undefined') {
            modifier = 1;
        }
        self.status.minimum += self.status.customerGain * modifier;
        if (self.status.minimum > self.status.customers) {
            self.status.customers = Math.min(self.status.minimum,
                self.status.capacity);
        }
        self.status.customerGain *= 1.75 * modifier;
    }

    var upgradeStock = function () {
        self.status.customerWealth += self.status.customerWealthGain;
        self.status.customerWealthGain *= 2.0;
        upgradeShoutStrength(0.5);
    }

    var upgradeShopSize = function (modifier) {
        if (typeof modifier === 'undefined') {
            modifier = 1;
        }
        var hasQueue = self.status.minimum > self.status.capacity;
        self.status.capacity += self.status.capacityGain;
        self.status.capacity *= modifier;
        self.status.capacity = Math.ceil(self.status.capacity);
        self.status.capacityGain *= 1.5;
        if (hasQueue) {
            self.status.customers = Math.min(self.status.minimum,
                self.status.capacity);
        }
    }

    var upgradeShopComfort = function () {
        self.status.customerDecayRate -= self.status.decayRateDecline;
        self.status.customerBuyRate += self.status.customerBuyRateGain;
        upgradeShoutStrength(0.5);
    }

    //////////////////////////////////////////////////
    ///                 BUTTONS                    ///
    //////////////////////////////////////////////////

    // Customers
    this.btnShout = createBigButton(0, "Go shout on the street", -1, undefined,
        function (upgrade) {
            self.status.customers += self.status.shoutStrength;
            if (self.status.customers > self.status.capacity) {
                self.status.customers = self.status.capacity;
            }
        });
    this.btnShout.bought = true;

    createSmallButton("Megaphone", 16, this.btnShout,
        "Let your voice be heard!\nIncreases the effect of your shouts.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeShoutStrength();
                ROY = ROY_MEGA;
            }
        });

    createSmallButton("Ghetto Blaster", 200, this.btnShout,
        "Nothing quite gets people's attention like a big-ass ghetto blaster right in front of your door.\nIncreases the effect of your shouts.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeShoutStrength();
            }
        });

    createSmallButton("Stereo Installation", 1000, this.btnShout,
        "Time to turn it up to eleven! Now they'll be able to hear you almost everywhere (within a 500m radius).\nIncreases the effect of your shouts.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeShoutStrength();
                ROY = ROY_MICRO;
            }
        });

    createSmallButton("Radio Commercial", 14000, this.btnShout,
        "Now the whole city will know about these magnificent deals!\nTurns your shouts into radio commercials and increases their effect.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                makesCommercials = true;
                self.btnShout.btn.text = "Launch a new commercial";
                self.btnCommercial.popOver.text = self.btnCommercial.popOver.text.replace("shouts", "commercials");
                upgradeShoutStrength();
            }
        });

    createSmallButton("TV Commercial", 35000, this.btnShout,
        "It'd be a shame if only your fellow citizens could enjoy these great deals. Time to educate the nation!\nIncreases the effect of your commercials.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeShoutStrength();
            }
        });

    createSmallButton("Online Commercial", 2500000, this.btnShout,
        "Time to go global! People all over the world will now learn about your great deals while watching Let's Plays on YouTube!\nIncreases the effect of your commercials.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeShoutStrength();
            }
        });

    // Minimum
    this.btnSign = createBigButton(1, "a shop sign", 6,
        "Helps people notice that this is actually a shop.\nIncreases the number of people that visit your shop without you having to shout at them.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeMinimumCustomers();
                self.sign = "small";
            }
        });

    createSmallButton("Bigger Sign", 12, this.btnSign,
        "For bigger people.\nIncreases the minimum number of people in your shop.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeMinimumCustomers();
            }
        });

    createSmallButton("Huge Sign", 24, this.btnSign,
        "This sign is huge. Ridiculously huge. If I had had more time, I'd have shown it on the left, but now you'll have to take my word for it.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeMinimumCustomers();
            }
        });

    createSmallButton("Neon Lights", 72, this.btnSign,
        "Add some more pizzazz to your sign! Who doesn't love neon? :D\nIncreases the minimum number of people in your shop at all time.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeMinimumCustomers();
                self.sign = "neon";
            }
        });

    createSmallButton("Red Carpet", 300, this.btnSign,
        "Receive your customers in style!\nIncreases the minimum number of people in your shop.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeMinimumCustomers(2);
                self.sign = "spotlight";
            }
        });

    createSmallButton("Spotlights", 110000, this.btnSign,
        "Let everybody know this is where the rich and famous do their shopping!\nIncreases the minimum number of people in your shop.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeMinimumCustomers(3);
                self.sign = "spotlight";
            }
        });

    createSmallButton("Billboard", 4500000, this.btnSign,
        "People can now see your shop from two cities over! Let everybody know where you're located.\nIncreases the minimum number of people in your shop.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeMinimumCustomers(4);
            }
        });

    // Comfort
    this.btnComfort = createBigButton(2, "an instore radio", 32,
        "People like shopping with music on the background. Increases people's shopping comfort and the effect of your shouting. If I had had more time, you'd now start hearing music.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeShopComfort();
            }
        });

    createSmallButton("Better Lighting", 750, this.btnComfort,
        "Your customers should be able to see what else they can buy in your store!\nIncreases your customers' shopping comfort and the effect of your shouting.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeShopComfort();
            }
        });

    createSmallButton("Redesign Store", 8192, this.btnComfort,
        "If you move everything around once in a while, people will stumble on things they didn't know they wanted while looking for what they did want to buy.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeShopComfort();
            }
        });

    this.btnCommercial = createSmallButton("Expensive Carpet", 16800, this.btnComfort, "Pamper your customers' feet and they will stay longer!\nIncreases your customers' shopping comfort and the effect of your shouts.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeShopComfort();
            }
        });

    createSmallButton("Modern Lighting", 85000, this.btnComfort,
        "For when classic lighting isn't good enough anymore! As long as your customers hang around for a little longer, it's fine by me.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeShopComfort();
            }
        });

    createSmallButton("Soothing Lounge Music", 175000, this.btnComfort,
        "\"Forget about all of your worries and just buy, buy, buy!\"\nIncreases your customers' shopping comfort.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeShopComfort();
            }
        });

    createSmallButton("Modernize Store", 6000000, this.btnComfort,
        "These old shelves just aren't cutting it anymore. Out with the old, in with the new! Upgrade everything.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeShopComfort();
                upgradeStock();
                upgradeShoutStrength();
                upgradeShopSize(0.95);
                upgradeMinimumCustomers(5);
            }
        });

    // Capacity
    this.btnSize = createBigButton(3, "more room", 128,
        "It's getting crowded in here.\nTime to increase your capacity!",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeShopSize();
            }
        });

    createSmallButton("Another Room", 800, this.btnSize,
        "You need space to place all this stuff!\nIncreases your capacity.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeShopSize();
            }
        });

    createSmallButton("... And Another Room", 35000, this.btnSize,
        "A lot of people want to buy your stuff! Time to expand again!\nIncreases your capacity.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeShopSize();
            }
        });

    createSmallButton("Another Branch", 80000, this.btnSize,
        "People are coming from all over the country to visit your store. Maybe you should open another branch?\nGreatly increases your capacity.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                isChain = true;
                self.btnSell.text = self.btnSell.text.replace("Shop", "Chain");
                upgradeShopSize(2);
            }
        });

    var btnShopName = createSmallButton("More Branches", 850000, this.btnSize,
        "People all over the world want to buy at @! Time to open some more branches!\nGreatly increases your capacity.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                isChain = true;
                self.btnSell.text = self.btnSell.text.replace("Shop", "Chain");
                upgradeShopSize(2);
            }
        });

    createSmallButton("Oversea Branches", 7500000, this.btnSize,
        "Now people on the other side of the ocean also can buy at one of your stores! Fantastic!\nGreatly increases your capacity.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                isChain = true;
                self.btnSell.text = self.btnSell.text.replace("Shop", "Chain");
                upgradeShopSize(2);
            }
        });

    createSmallButton("Online Store", 30000000, this.btnSize,
        "You're fully global now! This means everybody in the world can be your customer (as long as your server holds out of course).\nImmensely increases your capacity.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                isChain = true;
                self.btnSell.text = self.btnSell.text.replace("Shop", "Chain");
                upgradeShopSize(5);
                upgradeMinimumCustomers(10);
            }
        });

    // Wealth
    this.btnStock = createBigButton(4, "a larger stock", 512,
        "More stuff to buy = more people buying.\nIncreases the number of wealthy customers at your shop.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeStock();
            }
        });

    createSmallButton("Huge Stock", 8000, this.btnStock,
        "Look at all this stuff!\nIncreases the number of wealthy customers at your shop and the effect of your commercials.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeStock();
            }
        });

    createSmallButton("Even Larger Stock", 250000, this.btnStock,
        "Now it's just getting ridiculous.\nIncreases the number of wealthy customers at your shop and the effect of your commercials.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeStock();
            }
        });

    createSmallButton("Ridiculously Huge Stock", 1000000, this.btnStock,
        "What could anyone ever do with this much stuff? Increases the number of wealthy customers at your shop and the effect of your commercials.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeStock();
            }
        });

    createSmallButton("Warehouse", 5000000, this.btnStock,
        "Just look at all of this stuff to sell at a markup of 1000%! Increases the number of wealthy customers at your shop.",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeStock();
            }
        });

    createSmallButton("Offshore Bank Account", 25000000, this.btnStock,
        "Why pay taxes when you can move everything overseas? Increases the number of wealthy customers at your shop (to learn about your savings strategies).",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeStock();
            }
        });

    createSmallButton("Offshore Bank", 100000000, this.btnStock,
        "Why just open an account if you can open an entire bank? Greatly increases the number of wealthy customers at your shop (pure coincidence of course).",
        function (upgrade) {
            if (upgrade.buy(self.status)) {
                upgradeStock();
            }
        });

    //////////////////////////////////////////////////
    ///                  OTHER                     ///
    //////////////////////////////////////////////////
    this.btnSell = new Button(new Vector2(58, TOP_MARGIN + 4), new Vector2(100, 40),
        "#f6ba2f", "Shop Status", 20, ORIG_BTN_SELL_COLOUR,
        undefined, "#a67e20", 2);
    this.btnSell.onclick = function () {
        self.willDisappear();
        self.statusBox.isChain = isChain;
        self.statusBox.willAppear();
        self.statusBox.isVisible = true;
    }

    this.btnClear = new Button(new Vector2(SAVE_ICON_POS.x - 65, SAVE_ICON_POS.y), new Vector2(60, TOP_MARGIN - 6),
        "#f6ba2f", "Clear Data", 16, "#735716",
        undefined, "#a67e20", 2);
    this.btnClear.onclick = function () {
        var doIt = window.confirm("This will delete everything, including your global stats and shop name. Are you sure you want to do this?");
        if (doIt) {
            cookieManager.deleteEverything();
            location.reload();
        }
    }

    var boxSize = new Vector2(600, 300);

    this.statusBox = new StatusBox(
        new Vector2(400 - boxSize.x / 2, 225 - boxSize.y / 2), boxSize,
        this.status, "#ffe5a9", "#a67e20", 4, true,
        "\"@\" Status:", 32, "#000", "'Berkshire Swash', HelveticaNeue, sans-serif", 18, undefined, new Vector2(100, 40), 19);
    this.statusBox.onclose = function () {
        self.statusBox.willDisappear();
        self.statusBox.isVisible = false;
        self.willAppear();
    }

    this.statusBox.onsell = function () {
        var possSpeedUp = Math.floor(self.status.getSpeedUp(self.status.totalLoyalCustomers));
        var speedUpSentence = possSpeedUp > 1 ? "You'll receive a bonus speedup according to the number of loyal customers you have. Right now this bonus would be around x" + Math.round(self.status.getSpeedUp(self.status.totalLoyalCustomers)) + ". " : "";
        var doIt = window.confirm("Your loyal customers will stay with you, but you'll lose everything else. " + speedUpSentence + "Are you sure you want to do this?");
        if (doIt) {
            self.status.reset();
            self.status.numberOfShopsSold++;
            self.btnShout.reset();
            self.btnShout.bought = true;
            self.btnShout.upgrades[0].isVisible = true;
            self.sign = undefined;
            ROY = ROY_THUMB;
            self.btnSign.reset();
            self.btnComfort.reset();
            self.btnSize.reset();
            self.btnStock.reset();
            self.statusBox.onclose();
            cookieManager.saveEverything();
        }
    }

    function setShopName(name) {
        self.shopName = name;
        btnShopName.popOver.text = btnShopName.popOver.text.replace("@", name);
        self.statusBox.titleText = self.statusBox.titleText.replace("@", name);

        if (self.nameBox) {
            self.nameBox.willDisappear();
            self.nameBox = undefined;
            self.willAppear();
        }
        if (!cookieManager) {
            initiateCookieManager();
        }
    }

    this.getSaveData = function () {
        var data = [];
        data.push({
            name: "shopName",
            value: this.shopName
        });
        var royData = {
            name: "roy"
        };
        switch (ROY) {
        case ROY_THUMB:
            royData.value = "thumb";
            break;
        case ROY_MEGA:
            royData.value = "mega";
            break;
        case ROY_MICRO:
            royData.value = "micro";
            break;
        }
        data.push(royData);

        var signData = {
            name: "sign",
            value: this.sign
        };
        if (!this.sign) {
            signData.value = "none";
        }
        data.push(signData);

        data.push({
            name: "booleans",
            value: (isChain ? "T" : "F") + (makesCommercials ? "T" : "F")
        });

        return data;
    };

    this.loadSaveData = function () {
        setShopName(cookieManager.getObject("shopName"));

        var royValue = cookieManager.getObject("roy");
        switch (royValue) {
        case "thumb":
            ROY = ROY_THUMB;
            break;
        case "mega":
            ROY = ROY_MEGA;
            break;
        case "micro":
            ROY = ROY_MICRO;
            break;
        default:
            Log.warning("Unknown value for Roy: '" + royValue + "'", this);
            ROY = ROY_THUMB;
        }

        this.sign = cookieManager.getObject("sign");
        if (this.sign === "none") {
            this.sign = undefined;
        }

        isChain = cookieManager.getObject("booleans").substr(0, 1) === "T";
        makesCommercials = cookieManager.getObject("booleans").substr(1) === "T";

        if (isChain) {
            self.btnSell.text = self.btnSell.text.replace("Shop", "Chain");
        }

        if (makesCommercials) {
            self.btnShout.btn.text = "Launch a new commercial";
            self.btnCommercial.popOver.text = self.btnCommercial.popOver.text.replace("shouts", "commercials");
        }
    };

    function initiateCookieManager(loadData) {
        cookieManager = new CookieManager(function saveAllDataCB() {
            var saveData = [];
            saveData = saveData.concat(self.getSaveData());
            saveData = saveData.concat(self.status.getSaveData());
            saveData = saveData.concat(self.btnShout.getSaveData(false));
            saveData = saveData.concat(self.btnSign.getSaveData(true));
            saveData = saveData.concat(self.btnComfort.getSaveData(true));
            saveData = saveData.concat(self.btnSize.getSaveData(true));
            saveData = saveData.concat(self.btnStock.getSaveData(true));
            return saveData;
        }, function (doneSaving) {
            self.isSaving = !doneSaving;
        });

        if (loadData) {
            self.loadSaveData();
            self.status.loadSaveData(cookieManager);
            self.btnShout.loadSaveData(cookieManager);
            self.btnSign.loadSaveData(cookieManager);
            self.btnComfort.loadSaveData(cookieManager);
            self.btnSize.loadSaveData(cookieManager);
            self.btnStock.loadSaveData(cookieManager);
        }
    }

    if (CookieManager.prototype.hasSaveFile()) {
        console.log("Save file found!");
        initiateCookieManager(true);
    } else {
        console.log("No save found!");
        var boxSize = new Vector2(600, 300);
        this.nameBox = new InputBox(new Vector2(400 - boxSize.x / 2, 225 - boxSize.y / 2),
            boxSize, "#ffe5a9", "#a67e20", 4, true,
            "Welcome to " + ojglGameName + "!", 32, "#000", "'Berkshire Swash', HelveticaNeue, sans-serif", "Name your shop:", 16, "'Berkshire Swash', HelveticaNeue, sans-serif", 24, new Vector2(100, 40), 19,
            setShopName);

        // Put the circle mascot approx. in the lower right corner
        ROY_CIRCLE_POS.x = 400 + boxSize.x / 2 - 80;
        ROY_CIRCLE_POS.y = 225 + boxSize.y / 2 - 100;
    }
}

// Subclass the GameState class.
PlayState.prototype = GameState.prototype;

PlayState.prototype.update = function (deltaTime) {
    "use strict";

    this.status.update(deltaTime);
}

PlayState.prototype.render = function (screen) {
    "use strict";

    BACKGROUND.render(screen, BACKGROUND_POS);


    screen.renderText("" + ojglFPS, new Vector2(783, 435), 10, "#6580EF");
    screen.renderText("v." + ojglGameVersion, new Vector2(777, 446), 10, "#6580EF");

    var money = Math.ceil(this.status.money * 100) / 100;
    var colour = this.status.customers > this.status.capacity - 1 //
        ? "#70DF55" : (this.status.customers <= this.status.minimum //
            ? "#F44" : undefined);
    var extraText = this.status.minimum > this.status.capacity //
        ? ". There's a queue outside." //
        : "";
    var mSize = 32;

    screen.renderText("$ " + money,
        new Vector2(LEFT_MARGIN, TOP_MARGIN + mSize), mSize,
        undefined, undefined, undefined,
        "#000", undefined, 3, 4);

    screen.renderText("Customers in shop: " + Math.ceil(this.status.customers) //
        + "/" + this.status.capacity + extraText,
        new Vector2(LEFT_MARGIN, 87), 14,
        colour, undefined, undefined,
        "#000", new Vector2(1, 1), 1);

    var pulse = (new Date().getMilliseconds() / 500) - 1;
    pulse *= pulse;

    var INV_SPEED = 2000;
    var time = new Date().getTime() % INV_SPEED;
    var slowPulse = (time / (INV_SPEED / 2)) - 1;
    slowPulse *= slowPulse;

    // SIGN
    if (this.sign) {
        var shadowColour = undefined;
        var shadowOffset = undefined;
        var shadowBlur = undefined;
        var shadowWidth = undefined;

        // NEON
        if (this.sign === "neon" || this.sign === "spotlight") {
            shadowColour = "#F433FF";
            shadowBlur = 15 + 5 * pulse;
            shadowWidth = 1 + 2 * pulse;
        }

        SIGN.render(screen, SIGN_POS, SIGN_SCALE);
        screen.renderText(this.shopName, SIGN_NAME, 24,
            "#000", "'Berkshire Swash', HelveticaNeue, sans-serif",
            SIGN_WIDTH, shadowColour, shadowOffset, shadowBlur,
            shadowWidth, true, 3, "C", "C");
    }

    // SPOTLIGHT
    if (this.sign === "spotlight") {
        var oldAlpha = screen.alpha;
        screen.alpha = 0.4 + 0.4 * slowPulse;
        LIGHTBEAM.render(screen, LIGHTBEAM_POS, LIGHTBEAM_SCALE);
        screen.alpha = oldAlpha;

        SPOTLIGHT.render(screen, SPOTLIGHT_POS, SPOTLIGHT_SCALE);
    }

    // BUTTONS
    this.btnShout.render(screen, this.status, 1.0);
    this.btnSign.render(screen, this.status);

    if (this.btnSign.bought) {
        this.btnComfort.render(screen, this.status);

        if (this.btnComfort.bought) {
            this.btnSize.render(screen, this.status);

            if (this.btnSize.bought) {

                this.btnStock.render(screen, this.status);
            }
        }
    }

    var sellShadow = undefined;
    var possSpeedUp = Math.floor(this.status.getSpeedUp(this.status.totalLoyalCustomers));
    if (possSpeedUp >= 1.5 * this.status.speedUp) {
        this.btnSell.textColour = "#0A0";
        var g = ["2", "3", "4", "5", "6", "7", "8", "9", "A", "B"];
        g = g[Math.floor(g.length * slowPulse)];
        sellShadow = "#0" + g + "0";
    } else {
        this.btnSell.textColour = ORIG_BTN_SELL_COLOUR;
    }

    this.btnSell.render(screen, undefined, sellShadow, "#000");

    this.btnClear.render(screen, undefined, undefined, "#000");
    if (this.isSaving) {
        SAVE_ICON.render(screen, SAVE_ICON_POS);
    }

    if (typeof this.shopName === 'undefined' && this.nameBox) {
        this.nameBox.render(screen);
        ROY_CIRCLE.render(screen, ROY_CIRCLE_POS, ROY_CIRCLE_SCALE);
    } else {
        ROY.render(screen, ROY_POS, ROY_SCALE);
    }

    if (this.statusBox.isVisible) {
        this.statusBox.render(screen);
    }

    if (RENDER_DEBUG_TEXT) {
        screen.renderText("LD34 - 'Incremental Dare'",
            new Vector2(5, 20), 16,
            undefined, undefined, undefined,
            "#000", new Vector2(1, 1), 1);
        screen.renderText("Robin Arys", new Vector2(5, 36), 16,
            undefined, undefined, undefined,
            "#000", new Vector2(1, 1), 1);
        screen.renderText("FPS: " + ojglFPS, new Vector2(5, 52), 16,
            undefined, undefined, undefined,
            "#000", new Vector2(1, 1), 1);
    }
}

PlayState.prototype.willAppear = function () {
    "use strict";

    if (!this.nameBox) {
        this.btnShout.willAppear();
        this.btnSign.willAppear();
        this.btnStock.willAppear();
        this.btnSize.willAppear();
        this.btnComfort.willAppear();

        this.btnSell.willAppear();
        if (this.btnClear) {
            this.btnClear.willAppear();
        }
    } else {
        this.nameBox.willAppear();
    }
}

PlayState.prototype.willDisappear = function () {
    "use strict";

    this.btnShout.willDisappear();
    this.btnSign.willDisappear();
    this.btnStock.willDisappear();
    this.btnSize.willDisappear();
    this.btnComfort.willDisappear();

    this.btnSell.willDisappear();
    if (this.btnClear) {
        this.btnClear.willDisappear();
    }

    if (this.nameBox) {
        this.nameBox.willDisappear();
    }
}

PlayState.prototype.toString = function () {
    "use strict";

    return "PlayState";
}